import json
import logging
import os

import requests

from . import kafka_yaml
from .iamauthn import get_identity_v2_token
from .utils import profile

filename = os.path.basename(__file__)[:-3]

logger = logging.getLogger(filename)

content_type_application_json = "application/json"

# depending on the kafka object, there are different endpoints for different methods
# [0] = GET
# [1] = DELETE
kafka_endpoints = {
    "TOPIC": ["topics", "topics"],
    "CONSUMER_GROUP": ["groups", "group"],
    "TRANSACTIONAL_ID": ["transactions", "transaction"],
}


def init_envvars():
    dry_run = os.getenv("DRY_RUN", "false").lower() == "true"
    use_ssl = os.getenv("INSECURE", "true").lower() != "true"
    return (dry_run, use_ssl)


# entry for processing data, process the entire data yaml
def process_kobjs(config_yaml):
    logger.debug(f"process_kobjs(config_yaml: {config_yaml})")
    results = {}
    for cluster_alias in config_yaml["clusters"]:
        results[cluster_alias] = {}
        cluster = activate_cluster(cluster_alias)
        for key in config_yaml:
            if len([item for item in kobjs_methods.keys() if item[0] == key]) == 0:
                continue
            # process all kafka objects of the same type ie. {'topics','acls'}
            results[cluster_alias][key] = process_kobj(key, cluster, config_yaml[key])
    logger.debug(f"process_kobjs()->({results})")
    return results


def activate_cluster(cluster_alias):
    for cluster in kafka_yaml["clusters"]:
        if cluster_alias == cluster["alias"]:
            return cluster


def process_kobj(key, cluster, kobjs):
    logger.debug(f"process_kobj(key: {key}, cluster: {cluster}, kobjs: {kobjs})")
    results = []
    if kobjs is not None:
        for kobj in kobjs:
            try:
                result = kobjs_methods[(key, kobj["method"])](cluster, kobj)
            except KeyError as ke:
                result = {
                    "request": kobjs,
                    "errors": f"missing key {ke} in the request",
                }
            except TypeError as te:
                result = {
                    "request": kobjs,
                    "errors": f"{te}. try checking config yaml file for type errors",
                }
            except Exception as e:
                result = {"request": kobjs, "errors": str(e)}
            finally:
                if "errors" in result:
                    logger.error(result["errors"])
                results.append(result)
    logger.debug(f"process_kobj()->({results})")
    return results


@profile
def get_topic(cluster, topic):
    logger.debug(f"get_topic(cluster: {cluster}, topic: {topic})")
    payload = {}
    headers = {"Authorization": f"Bearer {get_identity_v2_token(cluster['authn'])}"}
    topic_url = f"{cluster['url']}/topic"
    get_topic_url = (
        f"{topic_url}/{topic['topicName']}" if "topicName" in topic else topic_url
    )
    return process_request(
        method="GET",
        url=get_topic_url,
        headers=headers,
        payload=payload,
    )


@profile
def create_topic(cluster, topic):
    logger.debug(f"create_topic(cluster: {cluster}, topic: {topic})")
    payload = {
        "topicName": topic["topicName"],
        "numPartitions": topic["numPartitions"],
        "replicationFactor": topic["replicationFactor"],
    }
    if "aclRequest" in topic:
        acl_request = topic["aclRequest"]
        payload["aclRequest"] = {
            "resourceType": acl_request["resourceType"]
            if "resourceType" in acl_request
            else "TOPIC",
            "permissions": acl_request["permissions"],
            "principal": acl_request["principal"],
            "resourceName": acl_request["resourceName"]
            if "resourceName" in acl_request
            else topic["topicName"],
        }
    if "configs" in topic:
        payload["configs"] = topic["configs"]
    headers = {
        "Content-Type": content_type_application_json,
        "Authorization": f"Bearer {get_identity_v2_token(cluster['authn'])}",
    }
    topic_url = f"{cluster['url']}/topic"
    return process_request(
        method="POST", url=topic_url, headers=headers, payload=payload
    )


@profile
def update_topic(cluster, topic):
    logger.debug(f"update_topic(cluster: {cluster}, topic: {topic})")
    payload = {"topicName": topic["topicName"]}
    if "numPartitions" in topic:
        payload["numPartitions"] = topic["numPartitions"]
    if "replicationFactor" in topic:
        payload["replicationFactor"] = topic["replicationFactor"]
    if "aclRequest" in topic:
        acl_request = topic["aclRequest"]
        payload["aclRequest"] = {
            "resourceType": acl_request["resourceType"]
            if "resourceType" in acl_request
            else "TOPIC",
            "permissions": acl_request["permissions"],
            "principal": acl_request["principal"],
            "resourceName": acl_request["resourceName"]
            if "resourceName" in acl_request
            else topic["topicName"],
        }
    if "configs" in topic:
        payload["configs"] = topic["configs"]
    headers = {
        "Content-Type": content_type_application_json,
        "Authorization": f"Bearer {get_identity_v2_token(cluster['authn'])}",
    }
    topic_url = f"{cluster['url']}/topic"
    return process_request(
        method="PUT",
        url=f"{topic_url}/{topic['topicName']}",
        headers=headers,
        payload=payload,
    )


@profile
def delete_topic(cluster, topic):
    logger.debug(f"delete_topic(cluster: {cluster}, topic: {topic})")
    payload = {}
    headers = {"Authorization": f"Bearer {get_identity_v2_token(cluster['authn'])}"}
    topic_url = f"{cluster['url']}/topic"
    return process_request(
        method="DELETE",
        url=f"{topic_url}/{topic['topicName']}",
        headers=headers,
        payload=payload,
    )


@profile
def get_acls(cluster, acl):
    logger.debug(f"get_acls(cluster: {cluster}, acl: {acl})")
    payload = {}
    headers = {"Authorization": f"Bearer {get_identity_v2_token(cluster['authn'])}"}
    resource_type = acl["resourceType"] if "resourceType" in acl else "TOPIC"
    acls_url = f"{cluster['url']}/acls"
    get_acls_url = f"{acls_url}/{kafka_endpoints[resource_type][0]}"
    if "resourceName" in acl:
        get_acls_url += f"/{acl['resourceName']}"
    return process_request(
        method="GET", url=get_acls_url, headers=headers, payload=payload
    )


@profile
def create_acls(cluster, acl):
    logger.debug(f"create_acls_for_topic(cluster: {cluster}, acl: {acl})")
    payload = {
        "resourceType": acl["resourceType"] if "resourceType" in acl else "TOPIC",
        "permissions": acl["permissions"],
        "principal": acl["principal"],
        "host": acl["host"] if "host" in acl else "*",
        "resourceName": acl["resourceName"],
    }
    headers = {
        "Content-Type": content_type_application_json,
        "Authorization": f"Bearer {get_identity_v2_token(cluster['authn'])}",
    }
    acls_url = f"{cluster['url']}/acls"
    return process_request(
        method="POST", url=acls_url, headers=headers, payload=payload
    )


@profile
def delete_acls(cluster, acl):
    logger.debug(f"delete_acls(cluster: {cluster}, acl: {acl})")
    payload = {}
    headers = {"Authorization": f"Bearer {get_identity_v2_token(cluster['authn'])}"}
    resource_type = acl["resourceType"] if "resourceType" in acl else "TOPIC"
    acls_url = f"{cluster['url']}/acls"
    delete_acl_url = (
        f"{acls_url}/{kafka_endpoints[resource_type][1]}/{acl['resourceName']}"
    )
    return process_request(
        method="DELETE",
        url=f"{delete_acl_url}?userName={acl['principal']}&permission=write",
        headers=headers,
        payload=payload,
    )


kobjs_methods = {
    ("topics", "GET"): get_topic,
    ("topics", "POST"): create_topic,
    ("topics", "PUT"): update_topic,
    ("topics", "DELETE"): delete_topic,
    ("acls", "GET"): get_acls,
    ("acls", "POST"): create_acls,
    ("acls", "DELETE"): delete_acls,
}


def process_request(method, url, headers, payload):
    logger.debug(
        f"process_request(method: {method}, url={url}, headers ={headers}, payload={payload})"
    )
    # init required environment variables
    (dry_run, use_ssl) = init_envvars()
    result = {"request": {"method": method, "url": url, "data": payload}}
    if not dry_run:
        try:
            response = requests.request(
                method,
                url,
                headers=headers,
                data=json.dumps(payload),
                verify=use_ssl,
            )
            response.raise_for_status()
            result["response"] = {
                "object": response.json(),
                "status_code": response.status_code,
                "reason": response.reason,
                "url": response.url,
            }
        except requests.HTTPError as he:
            result["response"] = {
                "object": response.json(),
                "status_code": response.status_code,
                "reason": response.reason,
                "url": response.url,
            }
            logger.error(he)
        except requests.exceptions.SSLError as ssle:
            result["errors"] = f"{ssle}. try with '-k/--insecure' argument"
        except Exception as e:
            result["errors"] = str(e)
        finally:
            result["request"] = {
                "method": method,
                "url": url,
                "data": payload,
            }
            if "errors" in result:
                logger.error(result["errors"])
    logger.debug(f"process_request()->({result})")
    return result
