#!/usr/bin/env python3

import json
import logging
import os
import time
import traceback
from datetime import datetime
from functools import wraps

import urllib3
import yaml
from dotenv import load_dotenv
from jsonformatter import JsonFormatter

filename = os.path.basename(__file__)[:-3]


# logging


# logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger(filename)


logging.getLogger("requests").setLevel(logging.WARNING)
logging.captureWarnings(True)

# prevent warning from insecure call
logging.getLogger("urllib3").setLevel(logging.WARNING)
urllib3.disable_warnings()


def set_log_level():
    log_level = os.getenv("LOG_LEVEL", "INFO")
    numeric_level = getattr(logging, log_level.upper(), None)
    if not isinstance(numeric_level, int):
        raise ValueError(f"Invalid log level: {log_level}")
    logging.basicConfig(level=numeric_level)


def init_logger(key):
    logger = logging.getLogger(key)
    logger.propagate = False
    formatter = JsonFormatter()
    handler = logging.StreamHandler()
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    return logger


# env & config


def init_env(required, default=[]):
    load_dotenv(verbose=logger.isEnabledFor(logging.DEBUG))
    set_log_level()
    # check env vars
    for var in required:
        if not os.getenv(var):
            raise ValueError(f"Envionment variable '{var}' not set")
    for var in default:
        if not os.getenv(var):
            logger.warning(
                f"Envionment variable '{var}' not set. Default value will be used."
            )


def load_config(env_var, file):
    env_config = os.getenv(env_var)
    if env_config:
        config = yaml.safe_load(env_config)
        logger.info(f"Config loaded from environment variable '{env_var}'")
        return config
    else:
        try:
            with open(file) as f:
                config_yaml = f.read()
                config_yaml_expanded = os.path.expandvars(config_yaml)
                if config_yaml != config_yaml_expanded:
                    logger.debug(f"Expanding environment variables in file: '{file}'")
                config = yaml.safe_load(config_yaml_expanded)
                logger.info(f"Config loaded from file: '{file}'")
                return config
        except FileNotFoundError:
            raise ValueError(
                f"No config found in environment variable '{env_var}' or file '{file}'"
            )


def write_log(content, filename):
    logger.debug(f"write_log(content: {content}, filename: {filename})")
    output_format = os.getenv("OUTPUT_FORMAT", "json")
    if output_format == "yaml":
        formatted_content = yaml.dump(content)
    else:
        formatted_content = json.dumps(content, indent=2)
    now = datetime.now().strftime("%Y%m%d_%H%M%S")
    with open(f"{filename}.{now}.log", "w") as f:
        f.write(formatted_content)
    logger.debug("write_log()->(0)")
    return 0


# error handling


def exception_as_dict(ex):
    return {
        "type": ex.__class__.__name__,
        "message": str(ex),
        "traceback": traceback.format_exc(),
    }


# profiling function


PROF_DATA = {}


def profile(fn):
    @wraps(fn)
    def with_profiling(*args, **kwargs):
        start_time = time.time()
        ret = fn(*args, **kwargs)
        elapsed_time = int((time.time() - start_time) * 1000)
        if fn.__name__ not in PROF_DATA:
            PROF_DATA[fn.__name__] = [0, []]
        PROF_DATA[fn.__name__][0] += 1
        PROF_DATA[fn.__name__][1].append(elapsed_time)
        return ret

    return with_profiling


def get_elapsed_time_prof_data(fn):
    return PROF_DATA[fn][1][-1]


# cache decorator for functions


def memoize(func):
    logger.debug("memoize({})".format(func))
    cache = dict()

    def memoized_func():
        if func in cache:
            logger.debug("memoized hit cache on ({})".format(func))
            return cache[func]
        logger.debug("memoized miss cache on ({})".format(func))
        result = func()
        cache[func] = result
        return result

    logger.debug("memoize()->({})".format(memoized_func))
    return memoized_func


def memoize_args(func):
    logger.debug("memoize_args({})".format(func))
    cache = dict()

    def memoized_func(*args):
        if args in cache:
            logger.debug("memoize_args hit cache on ({}({}))".format(func, args))
            return cache[args]
        logger.debug("memoize_args miss cache on ({}({}))".format(func, args))
        result = func(*args)
        cache[args] = result
        return result

    logger.debug("memoize_args()->({})".format(memoized_func))
    return memoized_func
